==> TundereBird-0.23.tar.bz2: TundereBird-0.23/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4/ImapFetchDataItem.cs <==
// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

namespace Smdn.Net.Imap4 {
  // combinable data item types:
  //   ImapCombinableDataItem
  //     => handles 'data item names' etc.
  //   * ImapFetchDataItem
  //       => handles 'message data item names or macro'
  //     ImapSearchCriteria
  //       => handles 'searching criteria'
  //     ImapSortCriteria
  //       => handles 'sort criteria'
  //     ImapStatusDataItem
  //       => handles 'status data item names'
  //     ImapStoreDataItem
  //       => handles 'message data item name' and 'value for message data item'

  // 6.4.5. FETCH Command
  //   message data item names or macro
  public class ImapFetchDataItem : ImapCombinableDataItem, IImapExtension {
    // ALL
    //    Macro equivalent to: (FLAGS INTERNALDATE RFC822.SIZE ENVELOPE)
    public static readonly ImapFetchDataItem All
      = new ImapFetchDataItem("ALL", true, "FLAGS", "INTERNALDATE", "RFC822.SIZE", "ENVELOPE");

    // FAST
    //    Macro equivalent to: (FLAGS INTERNALDATE RFC822.SIZE)
    public static readonly ImapFetchDataItem Fast
      = new ImapFetchDataItem("FAST", true, "FLAGS", "INTERNALDATE", "RFC822.SIZE");

    // FULL
    //    Macro equivalent to: (FLAGS INTERNALDATE RFC822.SIZE ENVELOPE
    //    BODY)
    public static readonly ImapFetchDataItem Full
      = new ImapFetchDataItem("FULL", true, "FLAGS", "INTERNALDATE", "RFC822.SIZE", "ENVELOPE", "BODY");

    // BODY
    //     Non-extensible form of BODYSTRUCTURE.
    public static readonly ImapFetchDataItem Body
      = new ImapFetchDataItem("BODY");

    // BODY[<section>]<<partial>>
    //     The text of a particular body section.  The section
    //     specification is a set of zero or more part specifiers
    //     delimited by periods.  A part specifier is either a part number
    //     or one of the following: HEADER, HEADER.FIELDS,
    //     HEADER.FIELDS.NOT, MIME, and TEXT.  An empty section
    //     specification refers to the entire message, including the
    //     header.
    public static ImapFetchDataItem BodyText()
    {
      return BodyCommon(false, null, null, null);
    }

    public static ImapFetchDataItem BodyText(string section)
    {
      return BodyCommon(false, section, null, null);
    }

    public static ImapFetchDataItem BodyText(int partialStart)
    {
      return BodyCommon(false, null, partialStart, null);
    }

    public static ImapFetchDataItem BodyText(int partialStart, int partialLength)
    {
      return BodyCommon(false, null, partialStart, partialLength);
    }

    public static ImapFetchDataItem BodyText(string section, int partialStart)
    {
      return BodyCommon(false, section, partialStart, null);
    }

    public static ImapFetchDataItem BodyText(string section, int partialStart, int partialLength)
    {
      return BodyCommon(false, section, partialStart, partialLength);
    }

    public static ImapFetchDataItem BodyText(string section, string @partial)
    {
      return BodyCommon(true, section, @partial);
    }

    // BODY.PEEK[<section>]<<partial>>
    //     An alternate form of BODY[<section>] that does not implicitly
    //     set the \Seen flag.
    public static ImapFetchDataItem BodyPeek()
    {
      return BodyCommon(true, null, null, null);
    }

    public static ImapFetchDataItem BodyPeek(string section)
    {
      return BodyCommon(true, section, null, null);
    }

    public static ImapFetchDataItem BodyPeek(long partialStart)
    {
      return BodyCommon(true, null, partialStart, null);
    }

    public static ImapFetchDataItem BodyPeek(long partialStart, long partialLength)
    {
      return BodyCommon(true, null, partialStart, partialLength);
    }

    public static ImapFetchDataItem BodyPeek(string section, long partialStart)
    {
      return BodyCommon(true, section, partialStart, null);
    }

    public static ImapFetchDataItem BodyPeek(string section, long partialStart, long partialLength)
    {
      return BodyCommon(true, section, partialStart, partialLength);
    }

    public static ImapFetchDataItem BodyPeek(string section, string @partial)
    {
      return BodyCommon(true, section, @partial);
    }

    // TODO: HEADER, HEADER.FIELDS, HEADER.FIELDS.NOT, MIME, and TEXT
    private static ImapFetchDataItem BodyCommon(bool peek, string section, long? partialStart, long? partialLength)
    {
      string part;

      if (partialStart == null) {
        part = null;
      }
      else {
        if (partialStart.Value < 0)
          throw new ArgumentOutOfRangeException("partialStart", "must be zero or positive number");

        if (partialLength == null) {
          part = string.Format("{0}", (long)partialStart);
        }
        else {
          if (partialLength.Value < 1)
            throw new ArgumentOutOfRangeException("partialLength", "must be non-zero positive number");

          part = string.Format("{0}.{1}", (long)partialStart, (long)partialLength);
        }
      }

      return BodyCommon(peek, section, part);
    }

    private static ImapFetchDataItem BodyCommon(bool peek, string section, string part)
    {
      if (string.IsNullOrEmpty(part))
        return new ImapFetchDataItem(string.Format("{0}[{1}]", peek ? "BODY.PEEK" : "BODY", section));
      else
        return new ImapFetchDataItem(string.Format("{0}[{1}]<{2}>", peek ? "BODY.PEEK" : "BODY", section, part));
    }

    // BODYSTRUCTURE
    //     The [MIME-IMB] body structure of the message.  This is computed
    //     by the server by parsing the [MIME-IMB] header fields in the
    //     [RFC-2822] header and [MIME-IMB] headers.
    public static readonly ImapFetchDataItem BodyStructure
      = new ImapFetchDataItem("BODYSTRUCTURE");

    // ENVELOPE
    //     The envelope structure of the message.  This is computed by the
    //     server by parsing the [RFC-2822] header into the component
    //     parts, defaulting various fields as necessary.
    public static readonly ImapFetchDataItem Envelope
      = new ImapFetchDataItem("ENVELOPE");

    // FLAGS
    //     The flags that are set for this message.
    public static readonly ImapFetchDataItem Flags
      = new ImapFetchDataItem("FLAGS");

    // INTERNALDATE
    //     The internal date of the message.
    public static readonly ImapFetchDataItem InternalDate
      = new ImapFetchDataItem("INTERNALDATE");

    // RFC822
    //     Functionally equivalent to BODY[], differing in the syntax of
    //     the resulting untagged FETCH data (RFC822 is returned).
    public static readonly ImapFetchDataItem Rfc822
      = new ImapFetchDataItem("RFC822");

    // RFC822.HEADER
    //     Functionally equivalent to BODY.PEEK[HEADER], differing in the
    //     syntax of the resulting untagged FETCH data (RFC822.HEADER is
    //     returned).
    public static readonly ImapFetchDataItem Rfc822Header
      = new ImapFetchDataItem("RFC822.HEADER");

    // RFC822.SIZE
    //     The [RFC-2822] size of the message.
    public static readonly ImapFetchDataItem Rfc822Size
      = new ImapFetchDataItem("RFC822.SIZE");

    // RFC822.TEXT
    //     Functionally equivalent to BODY[TEXT], differing in the syntax
    //     of the resulting untagged FETCH data (RFC822.TEXT is returned).
    public static readonly ImapFetchDataItem Rfc822Text
      = new ImapFetchDataItem("RFC822.TEXT");

    // UID
    //     The unique identifier for the message.
    public static readonly ImapFetchDataItem Uid
      = new ImapFetchDataItem("UID");

    /*
     * RFC 4551 - IMAP Extension for Conditional STORE Operation or Quick Flag Changes Resynchronization
     * http://tools.ietf.org/html/rfc4551
     */
    public static readonly ImapFetchDataItem ModSeq
      = new ImapFetchDataItem(ImapCapability.CondStore, "MODSEQ");

    private ImapFetchDataItem(ImapString macro, bool isMacro, params ImapString[] equivalentDataItemNames)
      : base(macro)
    {
      this.equivalent = equivalentDataItemNames;
    }

    private ImapFetchDataItem(params ImapString[] dataItemNames)
      : this(null, dataItemNames)
    {
    }

    private ImapFetchDataItem(ImapCapability requiredCapability, params ImapString[] dataItemNames)
      : base(dataItemNames)
    {
      this.equivalent = null;
      this.requiredCapability = requiredCapability;
    }

    public ImapFetchDataItem CombineWith(ImapFetchDataItem dataItem)
    {
      return this + dataItem;
    }

    public static ImapFetchDataItem Combine(ImapFetchDataItem x, ImapFetchDataItem y)
    {
      return x + y;
    }

    public static ImapFetchDataItem operator+ (ImapFetchDataItem x, ImapFetchDataItem y)
    {
      var xx = x.equivalent ?? x.Items;
      var yy = y.equivalent ?? y.Items;

      return new ImapFetchDataItem(x.requiredCapability ?? y.requiredCapability, GetCombinedItems(xx, yy));
    }

    public static ImapFetchDataItem FromUri(Uri uri)
    {
      return BodyPeek(ImapStyleUriParser.GetSection(uri), ImapStyleUriParser.GetPartial(uri));
    }

    ImapCapability IImapExtension.RequiredCapability {
      get { return requiredCapability; }
    }

    protected override ImapStringList GetCombined()
    {
      if (equivalent == null)
        // message data item names
        return ToParenthesizedString();
      else
        // macro
        return ToStringList();
    }

    private /*readonly*/ ImapString[] equivalent;
    private /*readonly*/ ImapCapability requiredCapability;
  }
}

==> TundereBird-0.24.tar.bz2: TundereBird-0.24/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4/ImapFetchDataItem.cs <==
// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

namespace Smdn.Net.Imap4 {
  // combinable data item types:
  //   ImapCombinableDataItem
  //     => handles 'data item names' etc.
  //   * ImapFetchDataItem
  //       => handles 'message data item names or macro'
  //     ImapSearchCriteria
  //       => handles 'searching criteria'
  //     ImapSortCriteria
  //       => handles 'sort criteria'
  //     ImapStatusDataItem
  //       => handles 'status data item names'
  //     ImapStoreDataItem
  //       => handles 'message data item name' and 'value for message data item'

  // 6.4.5. FETCH Command
  //   message data item names or macro
  public class ImapFetchDataItem : ImapCombinableDataItem, IImapExtension {
    // ALL
    //    Macro equivalent to: (FLAGS INTERNALDATE RFC822.SIZE ENVELOPE)
    public static readonly ImapFetchDataItem All
      = new ImapFetchDataItem("ALL", true, "FLAGS", "INTERNALDATE", "RFC822.SIZE", "ENVELOPE");

    // FAST
    //    Macro equivalent to: (FLAGS INTERNALDATE RFC822.SIZE)
    public static readonly ImapFetchDataItem Fast
      = new ImapFetchDataItem("FAST", true, "FLAGS", "INTERNALDATE", "RFC822.SIZE");

    // FULL
    //    Macro equivalent to: (FLAGS INTERNALDATE RFC822.SIZE ENVELOPE
    //    BODY)
    public static readonly ImapFetchDataItem Full
      = new ImapFetchDataItem("FULL", true, "FLAGS", "INTERNALDATE", "RFC822.SIZE", "ENVELOPE", "BODY");

    // BODY
    //     Non-extensible form of BODYSTRUCTURE.
    public static readonly ImapFetchDataItem Body
      = new ImapFetchDataItem("BODY");

    // BODY[<section>]<<partial>>
    //     The text of a particular body section.  The section
    //     specification is a set of zero or more part specifiers
    //     delimited by periods.  A part specifier is either a part number
    //     or one of the following: HEADER, HEADER.FIELDS,
    //     HEADER.FIELDS.NOT, MIME, and TEXT.  An empty section
    //     specification refers to the entire message, including the
    //     header.
    public static ImapFetchDataItem BodyText()
    {
      return BodyCommon(false, null, null);
    }

    public static ImapFetchDataItem BodyText(string section)
    {
      return BodyCommon(false, section, null);
    }

    public static ImapFetchDataItem BodyText(long partialStart, long partialLength)
    {
      return BodyCommon(false, null, new ImapPartialRange(partialStart, partialLength));
    }

    public static ImapFetchDataItem BodyText(ImapPartialRange @partial)
    {
      return BodyCommon(false, null, @partial);
    }

    public static ImapFetchDataItem BodyText(string section, long partialStart, long partialLength)
    {
      return BodyCommon(false, section, new ImapPartialRange(partialStart, partialLength));
    }

    public static ImapFetchDataItem BodyText(string section, ImapPartialRange @partial)
    {
      return BodyCommon(true, section, @partial);
    }

    // BODY.PEEK[<section>]<<partial>>
    //     An alternate form of BODY[<section>] that does not implicitly
    //     set the \Seen flag.
    public static ImapFetchDataItem BodyPeek()
    {
      return BodyCommon(true, null, null);
    }

    public static ImapFetchDataItem BodyPeek(string section)
    {
      return BodyCommon(true, section, null);
    }

    public static ImapFetchDataItem BodyPeek(long partialStart, long partialLength)
    {
      return BodyCommon(true, null, new ImapPartialRange(partialStart, partialLength));
    }

    public static ImapFetchDataItem BodyPeek(ImapPartialRange @partial)
    {
      return BodyCommon(true, null, @partial);
    }

    public static ImapFetchDataItem BodyPeek(string section, long partialStart, long partialLength)
    {
      return BodyCommon(true, section, new ImapPartialRange(partialStart, partialLength));
    }

    public static ImapFetchDataItem BodyPeek(string section, ImapPartialRange @partial)
    {
      return BodyCommon(true, section, @partial);
    }

    // TODO: HEADER, HEADER.FIELDS, HEADER.FIELDS.NOT, MIME, and TEXT
    private static ImapFetchDataItem BodyCommon(bool peek, string section, ImapPartialRange @partial)
    {
      if (@partial == null)
        return new ImapFetchDataItem(string.Format("{0}[{1}]", peek ? "BODY.PEEK" : "BODY", section));
      else if (@partial.IsLengthSpecified)
        return new ImapFetchDataItem(string.Format("{0}[{1}]{2}", peek ? "BODY.PEEK" : "BODY", section, @partial.ToString()));
      else
        throw new ArgumentException("length must be specified", "partial");
    }

    // BODYSTRUCTURE
    //     The [MIME-IMB] body structure of the message.  This is computed
    //     by the server by parsing the [MIME-IMB] header fields in the
    //     [RFC-2822] header and [MIME-IMB] headers.
    public static readonly ImapFetchDataItem BodyStructure
      = new ImapFetchDataItem("BODYSTRUCTURE");

    // ENVELOPE
    //     The envelope structure of the message.  This is computed by the
    //     server by parsing the [RFC-2822] header into the component
    //     parts, defaulting various fields as necessary.
    public static readonly ImapFetchDataItem Envelope
      = new ImapFetchDataItem("ENVELOPE");

    // FLAGS
    //     The flags that are set for this message.
    public static readonly ImapFetchDataItem Flags
      = new ImapFetchDataItem("FLAGS");

    // INTERNALDATE
    //     The internal date of the message.
    public static readonly ImapFetchDataItem InternalDate
      = new ImapFetchDataItem("INTERNALDATE");

    // RFC822
    //     Functionally equivalent to BODY[], differing in the syntax of
    //     the resulting untagged FETCH data (RFC822 is returned).
    public static readonly ImapFetchDataItem Rfc822
      = new ImapFetchDataItem("RFC822");

    // RFC822.HEADER
    //     Functionally equivalent to BODY.PEEK[HEADER], differing in the
    //     syntax of the resulting untagged FETCH data (RFC822.HEADER is
    //     returned).
    public static readonly ImapFetchDataItem Rfc822Header
      = new ImapFetchDataItem("RFC822.HEADER");

    // RFC822.SIZE
    //     The [RFC-2822] size of the message.
    public static readonly ImapFetchDataItem Rfc822Size
      = new ImapFetchDataItem("RFC822.SIZE");

    // RFC822.TEXT
    //     Functionally equivalent to BODY[TEXT], differing in the syntax
    //     of the resulting untagged FETCH data (RFC822.TEXT is returned).
    public static readonly ImapFetchDataItem Rfc822Text
      = new ImapFetchDataItem("RFC822.TEXT");

    // UID
    //     The unique identifier for the message.
    public static readonly ImapFetchDataItem Uid
      = new ImapFetchDataItem("UID");

    /*
     * RFC 4551 - IMAP Extension for Conditional STORE Operation or Quick Flag Changes Resynchronization
     * http://tools.ietf.org/html/rfc4551
     */
    public static readonly ImapFetchDataItem ModSeq
      = new ImapFetchDataItem(ImapCapability.CondStore, "MODSEQ");

    private ImapFetchDataItem(ImapString macro, bool isMacro, params ImapString[] equivalentDataItemNames)
      : base(macro)
    {
      this.equivalent = equivalentDataItemNames;
    }

    private ImapFetchDataItem(params ImapString[] dataItemNames)
      : this(null, dataItemNames)
    {
    }

    private ImapFetchDataItem(ImapCapability requiredCapability, params ImapString[] dataItemNames)
      : base(dataItemNames)
    {
      this.equivalent = null;
      this.requiredCapability = requiredCapability;
    }

    public ImapFetchDataItem CombineWith(ImapFetchDataItem dataItem)
    {
      return this + dataItem;
    }

    public static ImapFetchDataItem Combine(ImapFetchDataItem x, ImapFetchDataItem y)
    {
      return x + y;
    }

    public static ImapFetchDataItem operator+ (ImapFetchDataItem x, ImapFetchDataItem y)
    {
      var xx = x.equivalent ?? x.Items;
      var yy = y.equivalent ?? y.Items;

      return new ImapFetchDataItem(x.requiredCapability ?? y.requiredCapability, GetCombinedItems(xx, yy));
    }

    public static ImapFetchDataItem FromUri(Uri uri)
    {
      return BodyPeek(ImapStyleUriParser.GetSection(uri), ImapStyleUriParser.GetPartial(uri));
    }

    ImapCapability IImapExtension.RequiredCapability {
      get { return requiredCapability; }
    }

    protected override ImapStringList GetCombined()
    {
      if (equivalent == null)
        // message data item names
        return ToParenthesizedString();
      else
        // macro
        return ToStringList();
    }

    private /*readonly*/ ImapString[] equivalent;
    private /*readonly*/ ImapCapability requiredCapability;
  }
}
